var Loki = require('lokijs');
var Promise = require('bluebird');

var Keyring = function(options){
  if(typeof options === 'string'){
    options = {
      name: options
    };
  }
  if(typeof options !== 'object' || Array.isArray(options)){
    options = {};
  }
  if(!options.name){
    options.name = 'cryptchat-keys';
  }
  var col = new Loki.Collection(options.name);
  if(options.data && Array.isArray(options.data)){
    col.insert(options.data);
  }

  function add(key){
    if(Array.isArray(key)){
      var promises = [];
      for(var i=0;i<key.length;i++){
        promises.push(this.add(key[i]));
      }
      return Promise.all(promises);
    }

    if(typeof key.exportData !== 'undefined'){
      var keyring = this;
      return key.exportData().then(function(data){
        return keyring.add(data);
      });
    }
    return new Promise(function(resolve,reject){
      var res = col.insert(key);
      return resolve(res);
    });
  }

  function exportData(){
    return new Promise(function(resolve,reject){
      var keys = [];
      var data = col.find();
      for(var i=0;i<data.length;i++){
        var key = data[i];
        keys.push(key.keys);
      }
      return resolve({
        name: col.name,
        data: keys
      });
    });
  }

  function rawFind(query){
    return new Promise(function(resolve,reject){
      var res = col.find(query);
      return resolve(res);
    });
  }

  return {
    add: add,
    exportData: exportData,
    rawFind: rawFind
  };
};

module.exports = Keyring;
