var Keyring = require('../src/keyring');
var Crypto = require('cryptchat-crypto');
var assert = require('assert');

describe('Cryptchat.Keyring tests', function(){
  var password = 'password';
  var ring = new Keyring();

  describe('Basic I/O tests', function(){
    var key1,key2;
    it('Key insertion from exported data', function(){
      this.timeout(6000);
      return Crypto.pgp.generateKeypair({
        userId: {name: 'Test User 1', email: 'test1@user.com'},
        numBits: 1024,
        passphrase: password
      })
      .then(function(key){
        key1 = key;
        return key.exportData();
      })
      .then(function(data){
        return ring.add(data);
      })
      .then(function(instance){
        return assert.deepEqual(typeof instance.$loki,'number');
      });
    });

    it('Direct key insertion', function(){
      this.timeout(6000);
      return Crypto.pgp.generateKeypair({
        userId: {name: 'Test User 2', email: 'test2@user.com'},
        numBits: 1024,
        passphrase: password
      })
      .then(function(key){
        key2 = key;
        return ring.add(key);
      })
      .then(function(instance){
        var passed = true;
        if(typeof instance.exportData !== 'undefined'){
          passed = false;
        }
        return assert.deepEqual(passed, true);
      });
    });

    it('Multikey insertion', function(){
      return ring.add([key1,key2])
      .then(function(instances){
        var passed = true;
        if(!Array.isArray(instances)){
          passed = false;
        }else{
          for(var i=0;i<instances.length;i++){
            if(!instances[i].$loki || typeof instances[i].exportData !== 'undefined'){
              passed = false;
              i = instances.length;
            }
          }
        }
        return assert.deepEqual(passed, true);
      });
    });
  });

  describe('Load/Save tests', function(){
    var fullData,ring2;
    it('Export data', function(){
      return ring.exportData().then(function(exportData){
        fullData = exportData;
        data = fullData.data;
        var passed = true;
        if(!Array.isArray(data)){
          passed = false;
        }
        for(var i=0;i<data.length;i++){
          if(Object.keys(data[i]).length !== 2){
            passed = false;
            i = data.length;
          }else if(!data[i].public || !data[i].public){
            passed = false;
            i = data.length;
          }
        }
        if(typeof fullData !== 'object' || !fullData.data || typeof fullData.name !== 'string'){
          passed = false;
        }
        return assert.deepEqual(passed,true, 'Exported data does not match expected results');
      });
    });

    it('Import data', function(){
      var count1, count2;
      ring2 = new Keyring(fullData);
      return ring.rawFind().then(function(data){
        count1 = data.length;
      })
      .then(function(){
        return ring2.rawFind();
      })
      .then(function(data){
        count2 = data.length;
        return assert.deepEqual(count1, count2, 'Imported data count does not match original');
      });
    });

  });

});
